from typing import Any

class Flash:
    def ioctl(self, *args) -> Any: ...
    def readblocks(self, *args) -> Any: ...
    def writeblocks(self, *args) -> Any: ...

class PIO:
    IN_HIGH: int
    IN_LOW: int
    IRQ_SM0: int
    IRQ_SM1: int
    IRQ_SM2: int
    IRQ_SM3: int
    JOIN_NONE: int
    JOIN_RX: int
    JOIN_TX: int
    OUT_HIGH: int
    OUT_LOW: int
    SHIFT_LEFT: int
    SHIFT_RIGHT: int
    def add_program(self, *args) -> Any: ...
    def irq(self, *args) -> Any: ...
    def remove_program(self, *args) -> Any: ...
    def state_machine(self, *args) -> Any: ...

class StateMachine:
    def exec(self, *args) -> Any: ...
    def get(self, *args) -> Any: ...
    def active(self, *args) -> Any: ...
    def init(self, *args) -> Any: ...
    def irq(self, *args) -> Any: ...
    def put(self, *args) -> Any: ...
    def restart(self, *args) -> Any: ...
    def rx_fifo(self, *args) -> Any: ...
    def tx_fifo(self, *args) -> Any: ...



"""
multithreading support. See: https://docs.micropython.org/en/v1.17/library/_thread.html

|see_cpython_module| :mod:`python:_thread` https://docs.python.org/3/library/_thread.html .

This module implements multithreading support.

This module is highly experimental and its API is not yet fully settled
and not yet described in this documentation.
"""
from typing import Any

class LockType:
    def acquire(self, *args) -> Any: ...
    def locked(self, *args) -> Any: ...
    def release(self, *args) -> Any: ...

def allocate_lock(*args) -> Any: ...
def exit(*args) -> Any: ...
def get_ident(*args) -> Any: ...
def stack_size(*args) -> Any: ...
def start_new_thread(*args) -> Any: ...



from typing import Any

def crc8(*args) -> Any: ...
def readbit(*args) -> Any: ...
def readbyte(*args) -> Any: ...
def reset(*args) -> Any: ...
def writebit(*args) -> Any: ...
def writebyte(*args) -> Any: ...



from typing import Any

def __init__(*args) -> None: ...
def choice(*args) -> Any: ...
def getrandbits(*args) -> Any: ...
def randint(*args) -> Any: ...
def random(*args) -> Any: ...
def randrange(*args) -> Any: ...
def seed(*args) -> Any: ...
def uniform(*args) -> Any: ...



from _rp2 import *
from _typeshed import Incomplete

_PROG_DATA: Incomplete
_PROG_OFFSET_PIO0: Incomplete
_PROG_OFFSET_PIO1: Incomplete
_PROG_EXECCTRL: Incomplete
_PROG_SHIFTCTRL: Incomplete
_PROG_OUT_PINS: Incomplete
_PROG_SET_PINS: Incomplete
_PROG_SIDESET_PINS: Incomplete
_PROG_MAX_FIELDS: Incomplete

class PIOASMError(Exception): ...

class PIOASMEmit:
    labels: Incomplete
    prog: Incomplete
    wrap_used: bool
    sideset_count: int
    def __init__(
        self,
        *,
        out_init: Incomplete | None = ...,
        set_init: Incomplete | None = ...,
        sideset_init: Incomplete | None = ...,
        in_shiftdir: int = ...,
        out_shiftdir: int = ...,
        autopush: bool = ...,
        autopull: bool = ...,
        push_thresh: int = ...,
        pull_thresh: int = ...,
        fifo_join: int = ...,
    ) -> None: ...
    delay_max: int
    sideset_opt: Incomplete
    pass_: Incomplete
    num_instr: int
    num_sideset: int
    def start_pass(self, pass_) -> None: ...
    def __getitem__(self, key): ...
    def delay(self, delay): ...
    def side(self, value): ...
    def wrap_target(self) -> None: ...
    def wrap(self) -> None: ...
    def label(self, label) -> None: ...
    def word(self, instr, label: Incomplete | None = ...): ...
    def nop(self): ...
    def jmp(self, cond, label: Incomplete | None = ...): ...
    def wait(self, polarity, src, index): ...
    def in_(self, src, data): ...
    def out(self, dest, data): ...
    def push(self, value: int = ..., value2: int = ...): ...
    def pull(self, value: int = ..., value2: int = ...): ...
    def mov(self, dest, src): ...
    def irq(self, mod, index: Incomplete | None = ...): ...
    def set(self, dest, data): ...

_pio_funcs: Incomplete

def asm_pio(**kw): ...
def asm_pio_encode(instr, sideset_count): ...
